//=========================================================================
//
//	FILE:				CP_MouseEvents_View.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to display mouse events information. 
//	
//	HISTORY:			1st Version - August 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2005-08-30 22:33:04 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_MouseEvents_View
#define _H_CP_MouseEvents_View
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_ARGBColor.h"
#include "CP_View.h"

/*!
*	@class		CP_MouseEvents_View
*	@brief		CP_View subclass to display the current mouse coordinates when
*				the mouse is in the content area of this view.
*/
class CP_EXPORT CP_MouseEvents_View : public CPLAT::CP_View
{
public:
	enum
	{
		kMouseEventsView_NoEvent,
		kMouseEventsView_MouseEntered,
		kMouseEventsView_MouseMoved,
		kMouseEventsView_MouseExited
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MouseEvents_View(
												CPLAT::CP_View *inSuperview,
												const CPLAT::CP_Point& inUpperLeft,
												const CPLAT::CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_MouseEvents_View() throw();
	//@}
			
	//! @name Mouse Events
	//@{
	virtual	void						HandleMouseMoved(
												CPLAT::CP_Event_Base& inEvent,
												const CPLAT::CP_Point& inLocalMouseLocation );
	virtual	void						HandleMouseEntered( CPLAT::CP_Event_Base& inEvent );
	virtual	void						HandleMouseExited( CPLAT::CP_Event_Base& inEvent );
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CPLAT::CP_Canvas& inCanvas,
												CPLAT::CP_Region& inUpdateArea );
	//@}

	// class data
private:
	SInt16								fMouseEventStatus;
	CPLAT::CP_Point						fMouseLocation;
	CPLAT::CP_ARGBColor					fStrokeColor;			/*! Used to indicate when the mouse
																	has entered and exited this view. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif	// _H_CP_MouseEvents_View

