//=========================================================================
//
//	FILE:				CP_DebugMenuManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A singleton class to manage the Debug menu in a CPLAT
//						application.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_DebugMenuManager
#define _H_CP_DebugMenuManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Menu.h"
#include "CP_MenuBar.h"
#include "CP_NoCopy.h"
#include "CP_OSStatus.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_DebugMenuManager
*	@brief	A singleton classs to manage the Debug menu in a CPLAT application.
*	
*	This class manages a debug menu in your application, and provides methods to
*	assist in debuging CPLAT applications.
*/
class CP_EXPORT CP_DebugMenuManager : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DebugMenuManager();
	//! Destructor.
	virtual								~CP_DebugMenuManager();
	//@}

	//! @name Instance
	//@{
	static	CP_DebugMenuManager			*Instance();
	//@}

	//! @name Menu
	//@{
			void						LoadMenu( CP_MenuBar *inCurrentMenuBar );
	//@}

	//! @name Commands
	//@{
			OSStatus					HandleShowMessageCommand();
	//@}

protected:

private:
	//! @name Initialization
	//@{
			void						Init();
	//@}

	// class data
protected:
	static CP_DebugMenuManager			*fsInstance;	/*! The single instance of this class. */

	CP_Menu								*fDebugMenu;	/*! The CP_Menu that implements the Debug menu. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DebugMenuManager

