//=========================================================================
//
//	FILE:				CP_FontMenu.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage a font menu.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 233 $
//						Latest Checkout:  $Date: 2005-07-16 19:41:39 -0700 (Sat, 16 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_FontMenu
#define _H_CP_FontMenu
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Menu.h"
#include "CP_NoCopy.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_FontMenu
*	@brief		A class to manage a font menu.
*/
class CP_EXPORT CP_FontMenu : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FontMenu(
												bool inMenuBar,
												SInt16 inBeforeMenuID );
	//! Destructor.
	virtual								~CP_FontMenu() throw();
	//@}
			
	//! @name Instance
	//@{
			static CP_FontMenu			*Instance();
	//@}

	//! @name Getters
	//@{
			CP_Menu						*GetFontMenu()									{ return( fFontMenu ); }
	//@}

	//! @name Selection
	//@{
			CP_String					HandleMenuSelection( const CP_S_MenuSelection& inMenuSelection );
	//@}

protected:

	// class data
protected:
	static CP_FontMenu					*fsInstance;

	SInt16								fFontMenuID;
	CP_Menu								*fFontMenu;

	#if TARGET_OS_MAC
		MenuItemIndex					*fFontMenuHierarchicalItems;
		ItemCount						fNumHierarchicalItems;
		MenuItemIndex					fFontMenuCurrentMenuItem;
		MenuRef							fFontMenuCurrentMenuRef;
	#endif
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
			void						Init(
												bool inMenuBar,
												SInt16 inPosition );

			void						BuildFontMenuParentItemArray();
			CP_String					SelectAndGetFontName(
												MenuRef theMenu,
												MenuItemIndex theItem );
			MenuItemIndex				GetFontMenuParentItem( MenuRef inMenu );
#endif

#if TARGET_OS_WIN32
private:
			void						Init(
												bool inMenuBar,
												SInt16 inPosition );
#endif

#if TARGET_OS_LINUX
private:
			void						Init(
												bool inMenuBar,
												SInt16 inPosition );
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FontMenu

