//=========================================================================
//
//	FILE:				CP_OSIcon.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class for dealing with the native icon format on each
//						supported platform.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 231 $
//						Latest Checkout:  $Date: 2005-07-14 16:59:31 -0700 (Thu, 14 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_OSIcon
#define _H_CP_OSIcon
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Enums.h"
#include "CP_M_ReferenceCounted.h"
#include "CP_Rect.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_OSIcon
*	@brief		Class for dealing with the native icon format on each supported platform.
*/
class CP_EXPORT CP_OSIcon : CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_OSIcon();
										CP_OSIcon(
												CP_E_IconStyle inIconType,
												SInt32 inIconID );
										CP_OSIcon( const CP_OSIcon& inOSIcon );
	//! Destructor.
	virtual								~CP_OSIcon() throw();
	//@}
			
	//! @name Drawing
	//@{
			void						Draw(
												CP_Canvas& inDestCanvas,
												const CP_Rect& inBounds );
	//@}

	//! @name Getters
	//@{
			SInt16						Width();
			SInt16						Height();
			
										/*!
										*	Returns the OS type of the icon we are handling. This can be
										*	one of the following values:
										*		- kCP_IconStyle_UnknownType,
										*		- kCP_IconStyle_IconRef,
										*		- kCP_IconStyle_IconSuite,
										*		- kCP_IconStyle_ColorIcon,
										*		- kCP_IconStyle_DefaultSystemType
										*
										*	@exception	none
										*	
										*	@return		float const		Width of this object.
										*/
			CP_E_IconStyle				GetType()									{ return( fIconType ); }

										/*!
										*	Returns the OSColorIcon if this icon represents that type of icon,
										*	otherwise NULL is returned.
										*
										*	@exception	none
										*	
										*	@return		OSColorIcon		OSColorIcon for this icon.
										*/
			OSColorIcon					GetColorIcon()								{ return( fColorIcon ); }

										/*!
										*	Returns the OSIconRef if this icon represents that type of icon,
										*	otherwise NULL is returned.
										*
										*	@exception	none
										*	
										*	@return		OSIconRef		OSIconRef for this icon.
										*/
			OSIconRef					GetIconRef()								{ return( fIconRef ); }

										/*!
										*	Returns the OSIconSuite if this icon represents that type of icon,
										*	otherwise NULL is returned.
										*
										*	@exception	none
										*	
										*	@return		OSIconSuite		OSIconSuite for this icon.
										*/
			OSIconSuite					GetIconSuite()								{ return( fIconSuite ); }
	//@}

	//! @name Operators
	//@{
			CP_OSIcon&					operator = ( const CP_OSIcon& inOSIcon );
	//@}

	//! @name Setters
	//@{
			void						SetColorIcon( OSColorIcon inOSColorIcon );
			void						SetIconRef( OSIconRef inIconRef );
			void						SetIconSuite( OSIconSuite inIconSuite );
	//@}

protected:

	// class data
protected:
	CP_E_IconStyle						fIconType;		/* ! Type of icon we are representing. */
	
	OSIconRef							fIconRef;		/* ! OSIconRef, if this is an icon ref. */
	OSIconSuite							fIconSuite;		/* ! OSIconSuite, if this is an icon suite, this is the default type. */
	OSColorIcon							fColorIcon;		/* ! OSColorIcon, if this is an color icon, or CICN. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
										CP_OSIcon(
												OSType inCreator,
												OSType inIconType,
												SInt16 inVRefNum = kOnSystemDisk );
										CP_OSIcon(
												OSType inCreator,
												OSType inType,
												const CP_String& inName );

	IconRef								RegisterIcon(
												OSType inCreator,
												OSType inType,
												const CP_String& inName );
#endif	

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_OSIcon

