//=========================================================================
//
//	FILE:				CP_QuickDraw_GDI_Utility.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			QuickDraw / GDI utility routines used in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_QuickDraw_GDI_Utility
#define _H_CP_QuickDraw_GDI_Utility
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Rect.h"
#include "CP_Region.h"
#include "CP_RGBColor.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

#if defined(TARGET_OS_MAC) || defined(TARGET_OS_LINUX)
	#define SBT_NOBORDERS            0x0100
	#define SBT_POPOUT               0x0200
#endif

/*!
*	@namespace	CP_QD_GDI_Utility
*	@brief		A set of utility routines for QuickDraw / GDI.
*/
namespace CP_QD_GDI_Utility
{
		void							CP_DrawCharAt(
												CP_Canvas& inCanvas,
												char inChar,
												SInt16 inHorPosn,
												SInt16 inVerPosn );

		void 							CP_DrawTextAt(
												CP_Canvas& inCanvas,
												const CP_String& inText,
												SInt16 inHorPosn,
												SInt16 inVerPosn );

		void							CP_DrawStatusTextAt(
												CP_Canvas& inCanvas,
												const CP_String& inText,
												CP_Rect& inDestArea,
												SInt16 inFlags = 0,
												bool isActive = true );

		void 							CP_DrawTextAt(
												CP_Canvas& inCanvas,
												const char * inText,
												UInt16 inTextLength,
												SInt16 inHorPosn,
												SInt16 inVerPosn );
		void							CP_DrawTextAt(
												CP_Canvas& inCanvas,
												const CPUniChar *inText,
												UInt16 inTextLength,
												SInt16 inHorPosn,
												SInt16 inVerPosn );

		SInt16						CP_GetCharWidth(
											CP_Canvas& inCanvas,
											UInt16 inChar );
		SInt16						CP_GetStringWidth(
											CP_Canvas& inCanvas,
											const CP_String& inString );
		CP_Rect						CP_GetTextAreaBounds(
											CP_Canvas& inCanvas,
											CP_String& inText,
											SInt16 inLeading );
		SInt16 						CP_GetMaxStringWidth(
											CP_Canvas& inCanvas,
											const CP_String& inString );

		CP_String					ConvertLineEndings( const CP_String& inSourceString );

		#ifdef TARGET_OS_MAC
			void					CP_TintRegion_Mac(
											const CP_RGBColor& inOpColor,
											const CP_RGBColor& inForeColor,
											const CP_Region& inDestArea );

			void					GWorldImageBufferRelease(
											void *inInfo,
											const void *inData,
											size_t inSize );
		#endif
}	// namespace CP_QD_GDI_Utility

CPLAT_End_Namespace_CPLAT

#endif	// _H_QuickDraw_GDI_Utility
