//=========================================================================
//
//	FILE:				CP_StDialog.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A stack based class for display and handling of a dialog.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 180 $
//						Latest Checkout:  $Date: 2005-05-19 14:44:07 -0700 (Thu, 19 May 2005) $
//
//=========================================================================
#ifndef _H_CP_StDialog
#define _H_CP_StDialog
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DialogModality.h"
#include "CP_SignalSlot.h"
#include "CP_TimedTask.h"

CPLAT_Begin_Namespace_CPLAT

class CP_XMLDocument;

/*!
*	@class		CP_StDialog
*	@brief		A stack based class for display and handling of a dialog.
*
*				Note: CP_StDialogs are always stacked based, modal dialogs that run their own
*				event loop until the user causes the dialog to go away. If you need a dialog
*				that is modeless, then use the CP_Dialog class.
*/
class CP_EXPORT CP_StDialog : public CP_SigSlot::CP_SlotObject
{
public:
	enum
	{
		kCP_StDialog_DismissOK = 1,				/*!< User dismissed dialog by clicking default button. */
		kCP_StDialog_DismissCancel = 2			/*!< User dismissed dialog by clicking cancel button. */
	};
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	
										CP_StDialog(
												CP_XMLDocument& inXMLDoc,
												UInt32 inOKButtonID,
												UInt32 inCancelButtonID );
										CP_StDialog(
												SInt16 inXMLResID,
												UInt32 inOKButtonID,
												UInt32 inCancelButtonID );
										CP_StDialog(
												const CP_String& inNibName,
												const CP_String& inWindowName,
												UInt32 inOKButtonID,
												UInt32 inCancelButtonID );
	//! Destructor.
	virtual								~CP_StDialog() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the window object used to display the dialog.
										*	
										*	@exception	none
										*	
										*	@return		CP_Window *		Window displaying the dialog.
										*/
			CP_Window					*GetDialogWindow()							{ return( fDialogWindow ); }

										/*!
										*	Returns the result of the user dismissing the dialog.
										*	
										*	@exception	none
										*	
										*	@return		SInt16			Result of user dismissing dialog.
										*/
			SInt16						GetDismissResult()							{ return( fDismissResult ); }
	//@}

	//! @name Events
	//@{
			void						Run();
			void						Run( SInt32 inDuration );
	//@}

protected:
	//! @name Initialization
	//@{
	virtual void 						CommonInit(
												UInt32 inOKButtonID,
												UInt32 inCancelButtonID );
	//@}
			
private:
	//! @name Slots
	//@{
			void						HandleDismiss(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );
			void						HandleCancel(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );

			void						HandleTimer(
												CPLAT::CP_DataValueHolder& inDataValue,
												CPLAT::OSStatus& inOSStatus );
	//@}

	// class data
protected:
	CP_Window							*fDialogWindow;			/*! Dialog window. */
	CP_DialogModality					*fDialogModality;		/*! Modality of dialog. */
	
	SInt16								fDismissResult;			/*! Result of user dismissing dialog, */

	CP_TimedTask						*fDurationTimer;		/*! Timer used if dialog is displayed for a fixed length of time. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_StDialog

