//=========================================================================
//
//	FILE:				CP_TypeTraits.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definition of common MacOS types, used to provide a
//						type-safe way to using these types on some of the classes.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 256 $
//						Latest Checkout:  $Date: 2005-08-26 19:35:37 -0700 (Fri, 26 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_TypeTraits
#define _H_CP_TypeTraits
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Point.h"
#include "CP_Size.h"
#include "CP_UndoAction.h"

#if TARGET_OS_MAC
		// Apple
	#ifdef __CF_USE_FRAMEWORK_INCLUDES__
		#include <Carbon/Carbon.h>
	#else
		#include <Carbon.h>
	#endif
#endif	// TARGET_OS_MAC

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
	#if ! TARGET_RT_MAC_MACHO
			// defines for types used under MacOSX frameworks
		enum
		{
				// Carbon Events
			typeVoidPtr                   = 'void' /* void * (used for HIObject fun)*/
		};
	#endif	// TARGET_RT_MAC_MACHO
	
	template <SInt32 TYPE> struct CP_TypeTrait							{};

		// Core Types
	template <> struct CP_TypeTrait<typeSInt16>							{ typedef SInt32 Type; };
	template <> struct CP_TypeTrait<typeSInt32>							{ typedef SInt32 Type; };
	template <> struct CP_TypeTrait<typeUInt32>							{ typedef UInt32 Type; };
	template <> struct CP_TypeTrait<typeChar>							{ typedef char Type; };

		// Apple Types
	template <> struct CP_TypeTrait<typeBoolean>						{ typedef bool Type; };
	template <> struct CP_TypeTrait<typeControlRef>						{ typedef OSControl Type; };
	template <> struct CP_TypeTrait<typeDragRef>						{ typedef DragRef Type; };
	template <> struct CP_TypeTrait<typeQDPoint>						{ typedef Point Type; };
	template <> struct CP_TypeTrait<typeQDRectangle>					{ typedef Rect Type; };
	template <> struct CP_TypeTrait<typeWindowRef>						{ typedef OSWindow Type; };

	template <> struct CP_TypeTrait<kEventParamControlPart>				{ typedef ControlPartCode Type; };
	template <> struct CP_TypeTrait<kEventParamGrafPort>				{ typedef CGrafPtr Type; };
	template <> struct CP_TypeTrait<kEventParamCGContextRef>			{ typedef CGContextRef Type; };
	template <> struct CP_TypeTrait<kEventParamRgnHandle>				{ typedef RgnHandle Type; };
		
		// Carbon Events
	template <> struct CP_TypeTrait<typeHICommand>						{ typedef HICommand Type; };
	template <> struct CP_TypeTrait<typeVoidPtr>						{ typedef void * Type;	};

	template <> struct CP_TypeTrait<typeHIPoint>						{ typedef CP_Point Type; };

	#if TARGET_RT_MAC_MACHO
			// CFTypes
		template <> struct CP_TypeTrait<typeCFMutableArrayRef>			{ typedef CFMutableArrayRef Type; };

			// HITypes
		template <> struct CP_TypeTrait<typeHISize>						{ typedef CP_Size Type; };
		template <> struct CP_TypeTrait<typeHIObjectRef>				{ typedef HIObjectRef Type; };

			// since this is only defined in 10.4 and above headers
		template <> struct CP_TypeTrait<'shap'/*typeHIShapeRef*/>		{ typedef HIShapeRef Type; };
		
		template <> struct CP_TypeTrait<typeMouseTrackingRef>			{ typedef MouseTrackingRef Type; };
	#endif

		// CPLAT_Framework Types
	template <> struct CP_TypeTrait<kEventParamAction>					{ typedef CP_UndoAction * Type;	};
#endif	/// TARGET_OS_MAC

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TypeTraits

