//=========================================================================
//
//	FILE:				CP_FileStream.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CPLAT streaming class for reading and writing streamed
//						data to a CP_File object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_FileStream
#define _H_CP_FileStream
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_File.h"
#include "CP_InStream.h"
#include "CP_OutStream.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_FileOutstream
*	@brief		CP_OutStream class for writing data to a CP_File object.
*/
class CP_FileOutStream : public CP_OutStream
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileOutStream(
												CP_File& inOutFile,
												bool inHasHeader = true );
	//! Destructor.
	virtual								~CP_FileOutStream() throw();
	//@}

	//! @name Positioning
	//@{
	virtual UInt32						GetPosition() const;
	virtual void						SetPosition( UInt32 newPosition );
	//@}

protected:
	//! @name Writing
	//@{
	virtual void 						OnWriteBytes(
											const void *inDestBuffer,
											UInt32 inNumberBytes );

	virtual UInt32 						OnWriteData(
											const void *inDestBuffer,
											UInt32 inNumberBytes );
	//@}

protected:
	CP_File&							fFile;		/*! The CP_File object we are writing to. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

/*!
*	@class		CP_FileInstream
*	@brief		CP_InStream class for reading data from a CP_File object.
*/
class CP_FileInStream : public CP_InStream
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileInStream(
												CP_File& inFile,
												bool inHasHeader = true );
	//! Destructor.
	virtual								~CP_FileInStream() throw();
	//@}

	//! @name Getters
	//@{
	virtual UInt32						GetLength() const;
	//@}

	//! @name Positioning
	//@{
	virtual UInt32						GetPosition() const;
	virtual void						SetPosition( UInt32 inPosition );
	//@}

protected:
	//! @name Reading
	//@{
	virtual UInt32 						OnReadBytes(
												void *inDestBuffer,
												UInt32 inNumberBytes );
	//@}

protected:
	CP_File&							fFile;		/*! The CP_File object we are reading from. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileStream
