//=========================================================================
//
//	FILE:				CP_FileSystemSpec.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to manage the specification of a file in the
//						platforms file system.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 140 $
//						Latest Checkout:  $Date: 2005-04-09 22:02:13 -0700 (Sat, 09 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_FileSystemSpec
#define _H_CP_FileSystemSpec
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec_Imp_Gtk.h"
#include "CP_FileSystemSpec_Imp_Mac.h"
#include "CP_FileSystemSpec_Imp_Win.h"

#if TARGET_OS_MAC
		// Apple
	#if TARGET_RT_MAC_MACHO
		#include <Carbon/Carbon.h>
	#else
		#include <Files.h>
	#endif
#endif

#if TARGET_OS_WIN32
	#if CP_Optional_QuickTime_Windows
		#include <Files.h>
	#endif
#endif

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_FileSystemSpec
*	@brief		Class to manage the specification of a file in the platforms file system.
*	
*				This class is implemented for each of the supported platforms through a
*				CP_FileSystemSpec_Imp class.
*/
class CP_EXPORT CP_FileSystemSpec : public CP_FileSystemSpec_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileSystemSpec();
										CP_FileSystemSpec( const CP_FileSystemSpec& inFileSpec );
										CP_FileSystemSpec( const CP_String& inFullPath );
										CP_FileSystemSpec(
												const CP_FileSystemSpec& inParent,
												const CP_String& inName );
	//! Destructor.
	virtual								~CP_FileSystemSpec();
	//@}
				
	//! @name Getters
	//@{
			CP_FileSystemSpec			GetParentSpec() const;
			CP_String					GetFullPath() const;
	//@}

	//! @name operators
	//@{
			const CP_FileSystemSpec&	operator = ( const CP_FileSystemSpec& inFileSpec );
			bool						operator == ( const CP_FileSystemSpec& inFileSpec ) const;
			bool 						operator != ( const CP_FileSystemSpec& inFileSpec ) const;
	//@}

protected:

private:

	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileSystemSpec( FSRef inFileRef );
										CP_FileSystemSpec(
												const FSSpec& inFSSpec,
												CFStringEncoding inEncoding = ::CFStringGetSystemEncoding() );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	#if CP_Optional_QuickTime_Windows
										CP_FileSystemSpec( const FSSpec& inFSSpec );
	#endif
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_OS_LINUX
	typedef std::vector<CPLAT::CP_FileSystemSpec>																	CP_FileSystemSpec_Array;
	typedef std::vector<CPLAT::CP_FileSystemSpec>::iterator															CP_FileSystemSpec_Iterator;
	typedef std::vector<CPLAT::CP_FileSystemSpec>::reverse_iterator													CP_FileSystemSpec_ReverseIterator;
#else
	typedef std::vector<CPLAT::CP_FileSystemSpec, std::allocator<CPLAT::CP_FileSystemSpec> >						CP_FileSystemSpec_Array;
	typedef std::vector<CPLAT::CP_FileSystemSpec, std::allocator<CPLAT::CP_FileSystemSpec> >::iterator				CP_FileSystemSpec_Iterator;
	typedef std::vector<CPLAT::CP_FileSystemSpec, std::allocator<CPLAT::CP_FileSystemSpec> >::reverse_iterator		CP_FileSystemSpec_ReverseIterator;
#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileSystemSpec

