/*
=========================================================================

	FILE:		CP_M_XMLHandler.h
	
				Copyright  2000-2003. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	A class to read data in XML format.
	
	HISTORY:	1st Version - September 2000.

	CHANGES:	9/2000			First version

	VERSION:	Current Revision: $Revision: 2 $
				Latest Change On: $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $ by $Author: kenstah $

=========================================================================
*/
#ifndef _H_CP_M_XMLHandler
#define _H_CP_M_XMLHandler
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if defined(_MSC_VER)
	#pragma warning(disable : 4786)	
	#pragma warning(disable : 4138)	
#endif

	// CPLAT_Framework
#include "CP_String.h"
#include "CP_XMLAttributes.h"

	// expat
#include <expat.h>

	// Standard Library
#include <map>

CPLAT_Begin_Namespace_CPLAT

//========================================================================
// Class: CP_M_XMLHandler
//		Base mix-in class for objects that handle setting of information from
//			an xml document.
//========================================================================
//!  CP_M_XMLHandler
/*!
  Base mix-in class for objects that handle setting of information from an
  XML stream of data. 
*/
class CP_EXPORT CP_M_XMLHandler
{
public:
										// Construction / Destruction
									/*! Constructor.  */
										CP_M_XMLHandler()									{}
									/*! Destructor.  */
	virtual								~CP_M_XMLHandler()									{}
	
										// Called from the XML parser callbacks
									/*! Called from the XML parser when a start element is
									encountered in the XML document. Subclasses need to override this
									routine to provide handling of the passed in attributes. You need
									to return the CP_M_XMLHandler object that is going to receive the next
									OnStartElement call.  */
	virtual CP_M_XMLHandler				*OnStartElement(
												const XML_Char */*inElement*/,
												CP_XMLAttributes& /*inAttributes*/ )		{ return( NULL ); }

									/*! Called when an end element is encountered in the XML document.  */
	virtual bool						OnEndElement( const XML_Char */*inElement*/ )		{ return( true ); }

									/*! Called when you have character data in an XML element. Note: it is possible
									for the XML parser to send your data in various parts, and not all at once.
									\param s - the character data.
									\param inLength - the length of the data passed in.  */
	virtual void						OnCharacters(
					 							const XML_Char */*s*/,
					 							SInt16 /*inLength*/ )						{}

									/*! Called when a CDATA section is encountered.  */
	virtual	void						OnStartCData()										{}
									/*! Called when the previous CDATA section is ending.  */
	virtual	void						OnEndCData()										{}

									/*! Called when a processing instruction is encountered.  */
	virtual	void						OnProcessingInstruction(
												const XML_Char */*inTarget*/,
												const XML_Char */*inData*/ )				{}
											
									/*! Called when a comment is encountered.  */
	virtual	void						OnComment( const XML_Char */*inData*/ )				{}

		// class data members
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

//========================================================================
// Class: CP_XML_NullHandler
//		CP_M_XMLHandler class.
//========================================================================
//!  CP_XML_NullHandler
/*!
  A CP_M_XMLHandler object that is used when you need to return a CP_M_XMLHandler object,
  but don't need it to handle any events. This class is used so the parser callbacks
  have a handler to call as a default, but this handler doesn't do anything. 
*/
class CP_EXPORT CP_XML_NullHandler : public CP_M_XMLHandler
{
public:
	virtual CP_M_XMLHandler				*OnStartElement(
												const XML_Char */*inElement*/,
												CP_XMLAttributes& /*inAttributes*/ )		{ return( NULL ); }

	virtual bool						OnEndElement( const XML_Char */*inElement*/ )		{ return( true ); }

	virtual void						OnCharacters(
					 							const XML_Char */*s*/,
					 							SInt16 /*inLength*/ )						{};
	// class data members
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

/*
=========================================================================
REVISION HISTORY:

$Log: CP_M_XMLHandler.h,v $
Revision 1.1.1.1  2003/10/31 00:23:48  kenstah
Initial import.

=========================================================================
*/

#endif	// _H_CP_M_XMLHandler


