//=========================================================================
//
//	FILE:				CP_Application_Imp_Gtk.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation class for CP_Application using Linux
//						Gtk toolkit.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 194 $
//						Latest Checkout:  $Date: 2005-05-25 22:20:49 -0700 (Wed, 25 May 2005) $
//
//=========================================================================
#ifndef _H_CP_Application_Imp_Gtk
#define _H_CP_Application_Imp_Gtk
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_LINUX

	// CPLAT_Framework
#include "CP_EventHandler.h"
#include "CP_Enums.h"
#include "CP_OSStatus.h"
#include "CP_ScriptableObject.h"
#include "CP_SigSlot.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Application_Imp
*	@brief		Linux Gtk implementation of the CP_Application class.
*	
*				This object will implement the CP_Application class through
*				the Linux Gtk library for all of its event handling.
*/
class CP_EXPORT CP_Application_Imp : public CP_ScriptableObject, public CP_M_CommandHandler, public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Application_Imp();
	//! Destructor.
	virtual								~CP_Application_Imp() throw();
	//@}
	
	//! @name Instance
	//@{
	static CP_Application_Imp			*Instance();
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleQuitCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name Events
	//@{
	virtual	void						Run();
	//@}
			
	//! @name Exceptions
	//@{
	virtual void 						OnUncaughtException( const std::exception& e );
	//@}
			
	//! @name Application Status
	//@{
			CP_E_AppStatus				GetAppStatus();

										/*!
										*	Sets the status of our application from the passed in parameter.
										*	
										*	@param	inAppStatus			Flag to indicate the current condition of the application.
										*								These flags are defined in CP_Enums.h and indicate if the application
										*								is starting up, running, quitting, or quit.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetAppStatus( CP_E_AppStatus inAppStatus )						{ fAppStatus = inAppStatus; }
	//@}

protected:
	//! @name Events
	//@{
	virtual	bool						HandleQuitApplication()											{ return( false ); }
	virtual	void						HandleOpenApplication()											{}
	//@}

	//! @name Events
	//@{
			void						QuitApplication();
	//@}

private:
	//! @name Initialization
	//@{
			void						Init();
			void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	// class data
protected:
	static CP_Application_Imp			*fsInstance;		/*! Single instance of this class. */
								
	CP_E_AppStatus						fAppStatus;			/*! Current state of the application, startup, running, quitting or quit. */
	CP_EventHandler						fEventHandler;		/*! Event handling object for this class */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_LINUX

#endif	// _H_CP_Application_Imp_Gtk

