//=========================================================================
//
//	FILE:				CP_AE_AppleEvent.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Wrapper class for dealing with Apple Events.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 187 $
//						Latest Checkout:  $Date: 2005-05-22 13:06:02 -0700 (Sun, 22 May 2005) $
//
//=========================================================================
#ifndef _H_CP_AE_AppleEvent
#define _H_CP_AE_AppleEvent
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_AE_Descriptor.h"

CPLAT_Begin_Namespace_CPLAT

//! CP_E_SendEvent
/*! Determines what will happen when we try to send the AppleEvent. */
enum CP_E_SendEvent
{ 
	kCP_CheckAEM				= 0,	/*!< Checks at runtime to see if OK to send events to ourselves. Old versions leaked memory. */ 
	kCP_DontUseSelfSend			= 1,	/*!< Dont send the event to ourself. */
	kCP_SelfSendOK				= 2		/*!< Its OK to send Apple Events to ourselves. This is the default method, so we can record events. */
};

/*!
*	@class		CP_AE_AppleEvent 
*	@brief		Wrapper class for the MacOS AppleEvent.
*
*	@ingroup	AppleEvents
*/
class CP_EXPORT CP_AE_AppleEvent : public CP_AE_Descriptor
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AE_AppleEvent();
										CP_AE_AppleEvent( AppleEvent *inAppleEvent );
	//! Destructor.
	virtual								~CP_AE_AppleEvent();
	//@}

	//! @name Getters
	//@{
			void						GetParams(
												const AEKeyword inKeyword,
												const DescType inDesiredType,
												AEDesc& inOutOptionalParams ) const;
			void						GetOptionalParams(
												const AEKeyword inKeyword,
												const DescType inDesiredType,
												AEDesc& inOutOptionalParams ) const;

			void						GetParamPtr(
												const AEKeyword inKeyword,
												const DescType inDesiredType,
												DescType *inTypeCode,
												void *inDataPtr,
												const Size inMaximumSize,
												Size *inOutActualSize ) const;

			AEDesc			GetAttribute(
												const AEKeyword inKey,
												const DescType inDesiredType );
			SInt32						GetSInt32Attribute( const AEKeyword inKey );
	//@}

	//! @name Event makers
	//@{
			void						MakeAppleEvent(
												const AEEventClass inEventClass,
												const AEEventID inEventID,
												const AEDesc& inTarget,
												const SInt16 inReturnID = kAutoGenerateReturnID,
												const SInt32 inTransactionID = kAnyTransactionID );
			void						MakeAppleEvent(
												const AEEventClass inEventClass,
												const AEEventID inEventID,
												const ProcessSerialNumber& inPSN,
												const SInt16 inReturnID = kAutoGenerateReturnID,
												const SInt32 inTransactionID = kAnyTransactionID );
			void						MakeEventSelf(
												const AEEventClass inEventClass,
												const AEEventID inEventID,
												const SInt16 inReturnID = kAutoGenerateReturnID,
												const SInt32 inTransactionID = kAnyTransactionID );
			void						MakeEventSystem(
												const AEEventClass inEventClass,
												const AEEventID inEventID,
												const SInt16 inReturnID = kAutoGenerateReturnID,
												const SInt32 inTransactionID = kAnyTransactionID );
	//@}

	//! @name Senders
	//@{
			void						Send(
												AppleEvent *inOutReply,
												const AESendMode inSendMode,
												const AESendPriority inSendPriority = kAENormalPriority,
												const SInt32 inTimeOutInTicks = kAEDefaultTimeout,
												AEIdleUPP inIdleProc = NULL,
												AEFilterUPP inFilterProc = NULL );
			void						Send( AppleEvent *inFuture );
			void						Send_NoExecute();
			void						Send_NoReply();

			void						Suspend();
			void						Resume(
												AppleEvent *inReply,
												AEEventHandlerUPP inDispatcher,
												SInt32 inRefCon );
			void						SetCurrentEvent();
	//@}

	//! @name Setters
	//@{
			void						PutDescriptor(
												const AEKeyword inKey,
												AEDesc& inData );
			void						PutReturnDescriptor( AEDesc& inData );
			void						PutOptionalDescriptor(
												const AEKeyword inKey,
												AEDesc& inData );
			
			void						PutPtr(
												const AEKeyword inKey,
												const DescType inTypeCode,
												const void *inDataPtr,
												Size inDataSize );

			void						PutAttribute(
												const AEKeyword inKey,
												AEDesc& inAttribute );

			void						CheckRequiredParams() const;

			void						SpecifyThatParameterIsOptional( const AEKeyword inOptionalKeyword );
	//@}

protected:

	// class data members
protected:
	SInt16								fCanUseSelfSend;		/* ! Flag to indicate if we are working around a bug that
																	that causes memory leaks. */
};

CPLAT_End_Namespace_CPLAT

#endif	// __BUILD_MAC__

#endif	// H_CP_AE_AppleEvent
