//=========================================================================
//
//	FILE:				CP_CFDictionary.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle a MacOS CFDictionary.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 271 $
//						Latest Checkout:  $Date: 2005-09-06 19:24:58 -0700 (Tue, 06 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_CFDictionary
#define _H_CP_CFDictionary
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_CFMutableObject.h"
#include "CP_Debugging.h"
#include "CP_ErrorManager.h"
#include "CP_Exceptions.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class	CP_CFDictionary
*	\brief	A class to handle a MacOS CFDictionary.
*/
template <class TKey, class TValue>
class CP_EXPORT CP_CFDictionary : public CP_CFMutableObject<CFDictionaryRef, CFMutableDictionaryRef>
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_CFDictionary();
								CP_CFDictionary( const CP_CFDictionary& inOriginal );
								CP_CFDictionary(
										const TKey *inKeys,
										const TValue *inValues,
										CFIndex inItemCount,
										const CFDictionaryKeyCallBacks *inKeyCallBacks = NULL,
										const CFDictionaryValueCallBacks *inValueCallBacks = NULL,
										CFAllocatorRef inAllocator = NULL);
	//! Destructor.
	virtual						~CP_CFDictionary() throw();
	//@}
			
protected:

	// class data
private:	
	typedef CP_CFMutableObject<CFDictionaryRef, CFMutableDictionaryRef>		T_Base;
};

//=========================================================================
//	FUNCTION:	CP_CFDictionary
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
template <class TKey, class TValue>
CP_CFDictionary<TKey, TValue>::CP_CFDictionary()
{
}

//=========================================================================
//	FUNCTION:	CP_CFDictionary
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Copy constructor.
*
*	@param		inOriginal				The original CP_CFDictionary to copy
*										the fields from.
*	
*	@exception	none
*/
template <class TKey, class TValue>
CP_CFDictionary<TKey, TValue>::CP_CFDictionary( const CP_CFDictionary& inOriginal )
	: T_Base(inOriginal)
{
}

//=========================================================================
//	FUNCTION:	CP_CFDictionary
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Copy constructor.
*
*	@param		inKeys					List of keys for the dictionary items.
*	@param		inValues				List of values for the dictionary items.
*	@param		inItemCount				Number of items passed in.
*	@param		inKeyCallBacks			Dictionarykey callback for the keys.
*	@param		inValueCallBacks		Dictionarykey callback for the values.
*	@param		inAllocator				Allocator for the items in the dictionary.
*	
*	@exception	none
*/
template <class TKey, class TValue>
CP_CFDictionary<TKey, TValue>::CP_CFDictionary( const TKey *inKeys, const TValue *inValues, CFIndex inItemCount,
	const CFDictionaryKeyCallBacks *inKeyCallBacks, const CFDictionaryValueCallBacks *inValueCallBacks,
	CFAllocatorRef inAllocator )
{
	CFDictionaryRef		cfRef = NULL;
	
	cfRef = ::CFDictionaryCreate( inAllocator, (const void **) inKeys, (const void **) inValues,
		inItemCount, inKeyCallBacks, inValueCallBacks);
	CP_ThrowIfNULL(cfRef);
	
	AttachRef( cfRef, false );
}

//=========================================================================
//	FUNCTION:	~CP_CFDictionary
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
template <class TKey, class TValue>
CP_CFDictionary<TKey, TValue>::~CP_CFDictionary() throw()
{
}

CPLAT_End_Namespace_CPLAT

#endif  // TARGET_OS_MAC

#endif	// _H_CP_CFDictionary

