//=========================================================================
//
//	FILE:				CP_CFObject.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle a MacOS CFObject.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 131 $
//						Latest Checkout:  $Date: 2005-03-28 19:43:25 -0800 (Mon, 28 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_CFObject
#define _H_CP_CFObject
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_ErrorManager.h"
#include "CP_Exceptions.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class	CP_CFObject
*	\brief	A class to handle a MacOS CFObject.
*/
template <typename TCFRef>
class CP_EXPORT CP_CFObject
{
public:
			void				AttachRef(
										TCFRef inRef,
										bool inRetain );
	
			TCFRef				DetachRef();
			
			TCFRef				GetCFRef() const;

			bool				IsValid() const;

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_CFObject();
								CP_CFObject(
									TCFRef inRef,
									bool inRetain );
								CP_CFObject( const CP_CFObject& inOriginal );
	//! Destructor.
	virtual						~CP_CFObject() throw();
	//@}

private:
			void				Retain();
	
			void				Release();

	// class data
protected:
	TCFRef						fCFObject;		/*! The CFObject we manage. */
};

template <typename TCFRef>
CP_CFObject<TCFRef>::CP_CFObject()
{
	fCFObject = NULL;
}

template <typename TCFRef>
CP_CFObject<TCFRef>::CP_CFObject( TCFRef inRef, bool inRetain )
{
	fCFObject = inRef;
	
	if( inRetain )
		Retain();
}

template <typename TCFRef>
CP_CFObject<TCFRef>::CP_CFObject( const CP_CFObject& inOriginal )
{
	fCFObject = NULL;
	
	AttachRef( inOriginal.fCFObject, true );
}

template <typename TCFRef>
CP_CFObject<TCFRef>::~CP_CFObject() throw()
{
	Release();
}

template <typename TCFRef>
void CP_CFObject<TCFRef>::Retain()
{
	if( IsValid() )
		::CFRetain( fCFObject );
}

template <typename TCFRef>
void CP_CFObject<TCFRef>::Release()
{
	if( IsValid() )
		::CFRelease( fCFObject );
}

template <typename TCFRef>
bool CP_CFObject<TCFRef>::IsValid() const
{
	return( fCFObject != NULL );
}

template <typename TCFRef>
TCFRef CP_CFObject<TCFRef>::GetCFRef() const
{
	CP_ThrowIfNULL( fCFObject );
	
	return( fCFObject );
}

template <typename TCFRef>
void CP_CFObject<TCFRef>::AttachRef( TCFRef inRef, bool inRetain )
{
	if( inRef == fCFObject )
		return;
	
	Release();
	
	fCFObject = inRef;
	
	if( inRetain )
		Retain();
}

template <typename TCFRef>
TCFRef CP_CFObject<TCFRef>::DetachRef()
{
	TCFRef	detachedRef = fCFObject;
	
	fCFObject = NULL;
	
	return detachedRef;
}

CPLAT_End_Namespace_CPLAT

#endif  //TARGET_OS_MAC

#endif	// _H_CP_CP_CFObject

