//=========================================================================
//
//	FILE:				CP_HIObject.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_CP_M_SlotObject that provides the basics for dealing
//						with HIObjects in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 166 $
//						Latest Checkout:  $Date: 2005-05-03 16:59:08 -0700 (Tue, 03 May 2005) $
//
//=========================================================================
#ifndef _H_CP_HIObject
#define _H_CP_HIObject
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020

	// CPLAT_Framework
#include "CP_Event.h"
#include "CP_EventHandler.h"
#include "CP_OSStatus.h"

#ifdef __CF_USE_FRAMEWORK_INCLUDES__
	#include <Carbon/Carbon.h>
#endif	// __CF_USE_FRAMEWORK_INCLUDES__


#include "CP_SignalSlot.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_HIObject
*	@brief		CP_CP_M_SlotObject that provides the basics for dealing with HIObjects in CPLAT.
*	
*				This class is used to provide support and registration features for HIObject,
*				and allow them to use the CPLAT signal/slot mechanism for event handling.
*
*	@ingroup	UI
*/
class CP_HIObject : public CP_SigSlot::CP_SlotObject
{
public:
	//! @name Getters
	//@{
								/*!
								*	Return the CP_EventHandler object that is used to handle events for this class.
								*	
								*	@exception	none
								*	
								*	@return		CP_EventHandler&	The event handler to handle our events.
								*/
	virtual	CP_EventHandler&	GetEventHandler()									{ return( fEventHandler ); }

								/*!
								*	Return the EventTargetRef for this object.
								*	
								*	@exception	none
								*	
								*	@return		EventTargetRef		The event target for this class.
								*/
			EventTargetRef		GetEventTarget()									{ return( ::HIObjectGetEventTarget( fObjectRef ) ); }

								/*!
								*	Return the MacOS HIObjectRef for this object.
								*	
								*	@exception	none
								*	
								*	@return		HIObjectRef	const	The HIObjectRef for this object.
								*/
			HIObjectRef			GetObjectRef()	const									{ return( fObjectRef ); }
	//@
									
	//! @name Retention
	//@{
								/*!
								*	Mark this object for retention.
								*	
								*	@exception	none
								*	
								*	@return		void
								*/
			void				Retain()											{ ::CFRetain( fObjectRef ); }

								/*!
								*	Mark this object for release.
								*	
								*	@exception	none
								*	
								*	@return		void
								*/
			void				Release()											{ ::CFRelease( fObjectRef ); }
	//@}
	
	//! @name Debugging
	//@{
								/*!
								*	Print out debugging information for this object.
								*	
								*	@exception	none
								*	
								*	@return		void
								*/
			void				PrintDebugInfo()									{ return( ::HIObjectPrintDebugInfo( fObjectRef ) ); }
	//@}

protected:
	//! @name Typedefs
	//@{
	//! Callback used in the construction of this class.
		typedef ::OSStatus		(*ConstructProc)(
										HIObjectRef inBaseObject,
										CP_HIObject **outInstance );
	//@}
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_HIObject( HIObjectRef inObject );
	//! Destructor.
	virtual						~CP_HIObject() throw();
	//@}
	
	//! @name Initialization
	//@{
	static ::OSStatus			RegisterSubclass(
										CFStringRef inID,
										CFStringRef inBaseID,
										ConstructProc inProc );
	static EventRef				CreateInitializationEvent();
	
	virtual OSStatus			Initialize( CP_Event_Base& inEvent );
	//@}
	
	//! @name Debugging
	//@{
	virtual void				PrintDebugInfoSelf();
	//@}
	
private:
	//! @name Events
	//@{
	static pascal ::OSStatus	ObjectEventHandler_Private(
										EventHandlerCallRef inCallRef,
										EventRef inEvent,
										void *inUserData );
	//@}
	
	// class data
protected:
	HIObjectRef					fObjectRef;		/*! HIObject reference to the hi object. */

	CP_EventHandler				fEventHandler;	/*! Class that handles all events. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020

#endif // _H_CP_HIObject
