//=========================================================================
//
//	FILE:				CP_MacResources.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle MacOS resources.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 267 $
//						Latest Checkout:  $Date: 2005-08-31 19:33:20 -0700 (Wed, 31 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_MacResources
#define _H_CP_MacResources
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_HandleBlock.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class		CP_MacResources
*	\brief		A class to handle MacOS resources.
*/
class CP_EXPORT CP_MacResources
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MacResources(
												CP_FileSystemSpec& inFileSpec,
												bool inOpen = true,
												bool inOpenReadOnly = true,
												bool inCloseOnDestruction = true );
										CP_MacResources( const CP_MacResources& inMacResources );
	//! Destructor.
	virtual								~CP_MacResources() throw();
	//@}
	
	//! @name Current Resource File
	//@{
			void						SaveCurrent();
			void						MakeCurrent();
			void						RestoreCurrent();
	//@}

	//! @name Gettes
	//@{
										/*!
										*	Returns the resource ref number.
										*	
										*	@exception	none
										*	
										*	@return		SInt16			The resources ref number.
										*/
			SInt16						GetRefNumber()										{ return( fResourceRefNumber ); }
			SInt16						GetResourceCount( ResType inType );
			SInt16						GetTypesCount();
			
			bool						GetOneResource(
												ResType inType,
  												SInt16 inID,
  												bool inDetach,
												CP_HandleBlock& outResourceHandle );
			bool						GetOneIndResource(
												ResType inType,
  												SInt16 inID,
  												bool inDetach,
												CP_HandleBlock& outResourceHandle );
	//@}
			
	//! @name File
	//@{
			void						Open();
			void						Close();
	//@}
											
	//! @name operators
	//@{
			CP_MacResources&			operator =( const CP_MacResources& inMacResources );
	//@}

protected:

	// class data
protected:
	CP_FileSystemSpec					fFileSpec;					/*! File spec of the file being used. */
	
	SInt16								fResourceRefNumber;			/*! Resource reference number for the resource file. */
	SInt16								fSavedResRefNum;			/*! Saved resource reference number. */

	bool 								fOpenReadOnly;				/*! Flag to indicate if the file is opened read only. */
	bool 								fCloseOnDestruction;		/*! Flag to indicate if the file is closed on destruction. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_MacResources

