//=========================================================================
//
//	FILE:				CP_Menu_Imp_Mac.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			MacOS implementation of the CP_Menu class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 218 $
//						Latest Checkout:  $Date: 2005-06-22 23:43:56 -0700 (Wed, 22 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Menu_Imp_Mac
#define _H_CP_Menu_Imp_Mac
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_STLArrayDefines.h"
#include "CP_String.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_Menu_Imp;

typedef std::vector<CP_Menu_Imp *, std::allocator<CP_Menu_Imp *> >						CP_MenuImp_List;
typedef std::vector<CP_Menu_Imp *, std::allocator<CP_Menu_Imp *> >::iterator			CP_MenuImp_Iterator;

/*!
*	@class		CP_Menu_Imp
*	@brief		MacOS implementation of the CP_Menu class.
*/
class CP_EXPORT CP_Menu_Imp : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Menu_Imp();
										CP_Menu_Imp(
												SInt16 inMenuID,
												const CP_String& inMenuTitle );
										CP_Menu_Imp( const CP_String& inMenuTitle );
										CP_Menu_Imp(
												SInt16 inMenuID,
												const CP_String& inMenuTitle,
												OSMenu inOSMenu );
										CP_Menu_Imp( OSMenu inOSMenu );
	//! Destructor.
	virtual								~CP_Menu_Imp() throw();
	//@}
						
	//! @name Commands
	//@{
	virtual	CP_CommandNo				GetCommandNo( SInt16 inMenuItemIndex );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Return the MacOS platform menu reference.
										*
										*	@exception	none
										*	
										*	@return		OSMenu		MacOS menu reference.
										*/
			OSMenu						GetOSMenu()											{ return( fOSMenu ); }
			SInt16						GetMenuItemCount();
			CP_String					GetTitle() const;

										/*!
										*	Return the ID of this menu.
										*
										*	@exception	none
										*	
										*	@return		SInt16		ID of this menu.
										*/
			SInt16						GetID()												{ return( fMenuID ); }
			CP_CommandNo				GetCommand( SInt16 inItemIndex );
			SInt16						GetCommandItemIndex( const CP_CommandNo inCommand );
			CP_String					GetItemText( SInt16 inItemIndex );
	//@}
			
	//! @name Items
	//@{
			void						AppendItem(
												const CP_String& inItemText,
												CP_CommandNo inCommand );
			void						DeleteItem( UInt16 inItemIndex );
	//@}

	//! @name Setters
	//@{
			void						SetTitle( const CP_String& inTitle );

			void						SetItemMark(
												CP_CommandNo inCommand,
												SInt16 inItemMark,
												bool inHasMark );
			void						SetItemMark(
												SInt16 inIndex,
												SInt16 inItemMark,
												bool inHasMark );
	//@}

protected:			
	//! @name OSMenu
	//@{
			void						LoadOSMenu(
												SInt16 inMenuID,
												bool inPopup = false );
	//@}

private:

	//! @name Utility
	//@{
			void						RemoveQuitItem();
			void						RemovePreferenceItem();
	//@}

	// class data
protected:
	SInt16								fMenuID;		/*! Menu id of this menu. */
	OSMenu								fOSMenu;		/*! MacOS menu reference. */

	CP_MenuImp_List						fSubMenus;		/*! List of sub menus used in hier items. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_Menu_Imp_Mac

