//=========================================================================
//
//	FILE:				CP_ScriptableObject_Imp_AE.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			MacOS scriptable object support using Apple Events.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 187 $
//						Latest Checkout:  $Date: 2005-05-22 13:06:02 -0700 (Sun, 22 May 2005) $
//
//=========================================================================
#ifndef _H_CP_ScriptableObject_Imp_AE
#define _H_CP_ScriptableObject_Imp_AE
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
//#include "AppleEvent.h"
#include "CP_SignalSlot.h"

	// Standard Library
#include <map>
#if TARGET_CC_METROWERKS
	#include <hash_map>
#endif

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations'
class CP_AE_ObjectDispatcher;
class CP_ScriptableObject_Imp;

typedef CP_SigSlot::CP_Signal3<const AppleEvent&, AppleEvent&, SInt32>						CP_AppleEventSignal;

struct CP_ModelToken
{
	CP_ScriptableObject_Imp			*fTokenData;
};
typedef struct CP_ModelToken CP_ModelToken, *CP_ModelTokenPtr;
const DescType type_ModelToken																			= FOUR_CHAR_CODE('TOKE');

struct LessAppleEvent : public std::binary_function<SInt32, SInt32, bool>
{
								/*!
								*	operator () called when we need to compare the passed in EventTypeSpecs
								*	
								*	@exception	none
								*	
								*	@return		bool const		Returns true if lhs spec is less than rhs spec.
								*/
	bool						operator()(const SInt32& lhs, const SInt32& rhs) const
									{
										return( lhs < rhs );
									}
};

struct HashAppleEvent : public std::unary_function<SInt32, size_t>
{
								/*!
								*	operator () called when we need to generate the hash for the passed in EventTypeSpecs
								*	
								*	@exception	none
								*	
								*	@return		size_t const		Returns the hash key.
								*/
	size_t						operator()(const SInt32& key) const
								{
									return( (UInt32) key );
								}
};

struct EqualAppleEvent : public std::binary_function<EventTypeSpec, EventTypeSpec, bool>
{
								/*!
								*	operator () called when we need to compare the passed in EventTypeSpecs
								*	for equality.
								*	
								*	@exception	none
								*	
								*	@return		bool const		Returns true if lhs spec is equal to in rhs spec.
								*/
	bool						operator()(const SInt32& lhs, const SInt32& rhs) const
								{
									return( lhs == rhs );
								}
};

#if TARGET_CC_METROWERKS
		// prefer hash_map if available for a little more speed
	typedef Metrowerks::hash_map<SInt32, CP_AppleEventSignal *, HashAppleEvent, EqualAppleEvent >				CP_AppleEventHandler_Map;
	typedef Metrowerks::hash_map<SInt32, CP_AppleEventSignal *, HashAppleEvent, EqualAppleEvent >::iterator		CP_AppleEventHandler_Iterator;
#else
	typedef std::map<SInt32, CP_AppleEventSignal *, LessAppleEvent >											CP_AppleEventHandler_Map;
	typedef std::map<SInt32, CP_AppleEventSignal *, LessAppleEvent >::iterator									CP_AppleEventHandler_Iterator;
#endif

typedef std::pair<const SInt32, CP_AppleEventSignal *>															CP_AppleEventHandler_Entry;

class CP_EXPORT CP_ScriptableObject_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_ScriptableObject_Imp() throw();
	//@}

	//! @name Class
	//@{
	virtual DescType					GetAppleClass() const = 0;
	//@}

	//! @name AppleEvent Handlers
	//@{
											// Main AppleEvent handling routine, subclasses need
											// to override and implement this method
	virtual	void						HandleAppleEventSelf(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply,
												SInt32 inRefCon );
	virtual void						HandleAppleEvent(
												const AppleEvent& inAppleEvent,
												AppleEvent& outAEReply,
												AEDesc& outResult,
												SInt32 inRefCon );
	//@}

	//! @name Containers
	//@{
	static void							SetDefaultObject( CP_ScriptableObject_Imp *inDefaultObject );
	static CP_ScriptableObject_Imp		*GetDefaultObject()																{ return( gDefaultObject ); }
	
	static CP_ScriptableObject_Imp		*GetScriptableObjectFromToken( const AEDesc &inToken );

			CP_ScriptableObject_Imp		*GetSuperScriptableObject() const												{ return( fSuperScriptableObject ); }
	virtual void						SetSuperScriptableObject( CP_ScriptableObject_Imp *inSuperScriptableObject )	{ fSuperScriptableObject = inSuperScriptableObject; }

			CP_ScriptableObject_Imp		*GetDefaultSubScriptableObject() const											{ return( fDefaultSubScriptableObject ); }
			void						SetDefaultSubScriptableObject( CP_ScriptableObject_Imp *inObject )				{ fDefaultSubScriptableObject = inObject; }

			CP_ScriptableObject_Imp		*GetDefaultSuperScriptableObject() const										{ return( fDefaultSuperScriptableObject ); }
			void						SetDefaultSuperScriptableObject( CP_ScriptableObject_Imp *inObject )			{ fDefaultSuperScriptableObject = inObject; }
	//@}

	//! @name Comparison
	//@{
	virtual	bool						CompareObjects(
												const DescType inOperation,
												const CP_ScriptableObject_Imp& toWhat );

	//@}

	//! @name Contained Objects
	//@{
	virtual	SInt32						CountContainedObjects( const DescType inType );

	virtual	void						GetContainedObject(
												const DescType inDesiredType,
												const DescType inKeyForm,
												const AEDesc& inKeyData,
												AEDesc& outToken );

	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByName(
												const DescType inDesiredClass,
												const CP_String& inName );
	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByPosition(
												DescType inDesiredClass,
												SInt32 inPosition );
	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByUniqueID(
												DescType inDesiredClass,
												SInt32 inID );
	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByRelativePosition(
												DescType inDesiredClass,
												SInt32 inPosition );
	//@}

	//! @name Event Handlers
	//@{
	virtual	CP_ScriptableObject_Imp		*DoAE_CreateElement(
											const DescType inDesiredClass,
											const DescType inInsertLocation,
											const AppleEvent& inEvent,
											AppleEvent& inOutAEReply );
	//@}
													
	//! @name EventHandlers List
	//@{
			void						AddEvent( SInt32 inEventID, CP_AppleEventSignal *inEventSignal );
			void						RemoveEvent( SInt32 inEventID );

			bool						HasHandlerForEvent( SInt32 inEventID );

			CP_AppleEventSignal			*FindHandlerForEvent( SInt32 inEventID );
	//@}

	//! @name Token
	//@{
	virtual void						GetScriptableObjectToken(
												DescType inModelID,
												DescType inKeyForm,
												const AEDesc& inKeyData,
												AEDesc& outToken );

			void						PutInToken(
												CP_ScriptableObject_Imp	*inScriptableObject,
												AEDesc& outToken ) const;
	//@}

	//! @name Property
	//@{
			CP_ScriptableObject_Imp		*GetScriptableObjectProperty( DescType inProperty ) const;

	virtual	void						DoAE_GetData(
												const DescType inPropertyType,
												const AEDesc& inValue,
												AppleEvent& inOutReply );

	virtual	void						DoAE_SetData(
												const DescType inPropertyType,
												const AEDesc& inValue,
												AppleEvent& inOutReply );
	//@}

	//! @name Sending
	//@{
			void						SendAppleEvent_Self(
												const AEEventClass inEventClass,
												const AEEventID inEventID,
												const bool inDoExecute );
	//@}

	//! @name operator
	//@{
	CP_ScriptableObject_Imp&			operator =( const CP_ScriptableObject_Imp& inObject );
	//@}

	//! @name Specifiers
	//@{
	virtual	void						MakeSpecifier( AEDesc& inOutSpecifier );
	//@}

	//! @name Tester
	//@{
	virtual	void						GotRequiredParams( const AppleEvent& inAppleEvent );
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ScriptableObject_Imp();
										CP_ScriptableObject_Imp( CP_ScriptableObject_Imp *inObjectContainer );
	//@}

	//! @name Initialization
	//@{
	virtual	void						Install( CP_AE_ObjectDispatcher *inDispatcher = NULL );
	//@}

	//! @name AppleEventHandlers
	//@{
	virtual	void						DoAE_Activate(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Clone(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Close(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Delete(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_DoObjectsExist(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Move(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Open(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Print(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Save(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	virtual	void						DoAE_Revert(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	//@}

	//! @name Containers
	//@{
	virtual	void						GetContainedObject_FormAbsolutePosition(
												const DescType inDesiredClass,
												const DescType inForm,
												const AEDesc& inSelectionData,
												AEDesc& outToken  );
	virtual	void						GetContainedObject_FormName(
												const DescType inDesiredClass,
												const DescType inForm,
												const AEDesc& inSelectionData,
												AEDesc& outToken  );
	virtual	void						GetContainedObject_FormUniqueID(
												const DescType inDesiredClass,
												const DescType inForm,
												const AEDesc& inSelectionData,
												AEDesc& outToken  );
	virtual	void						GetContainedObject_FormRelativePosition(
												const DescType inDesiredClass,
												const DescType inForm,
												const AEDesc& inSelectionData,
												AEDesc& outToken  );
	virtual	void						GetContainedObject_FormPropertyID(
												const DescType inDesiredClass,
												const DescType inForm,
												const AEDesc& inSelectionData,
												AEDesc& outToken  );
	virtual	void						GetContainedObject_OtherForm(
												const DescType inDesiredClass,
												const DescType inForm,
												const AEDesc& inSelectionData,
												AEDesc& outToken  );
	//@}

	//! @name Specifier
	//@{
	virtual	void						MakeSelfSpecifier(
												AEDesc& inContainerSpec,
												AEDesc& inOutSelfSpec );
	//@}

	//! @name Getters
	//@{
	virtual	SInt32						GetPositionOfSubObject(
												const DescType inObjectID,
												const CP_ScriptableObject_Imp *inObject ) const;
	//@}

private:
	//! @name Event Handlers
	//@{
			void						HandleActivateEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleCloneEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleCloseEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleDeleteEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleDoObjectsExistEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleGetDataEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleMoveEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleOpenEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandlePrintEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleSaveEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleSetDataEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
			void						HandleRevertEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	//@}

	// class data members
protected:
	static CP_ScriptableObject_Imp		*gDefaultObject;					/*! The default object that events will be dispatched to. */

	static bool							gInited;							/*! Flag to indicate if this class has been initialized. */

	CP_AppleEventHandler_Map			fEventHandlers;						/*! Map that contains all of the registered handlers. */
	
	CP_ScriptableObject_Imp				*fSuperScriptableObject;			/*! Scriptable object that is our container. */

	CP_ScriptableObject_Imp				*fDefaultSubScriptableObject;		/*! Default scriptable object that is contained by this object. */
	CP_ScriptableObject_Imp				*fDefaultSuperScriptableObject;		/*! Default scriptable object that contains this object. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_ScriptableObject_Imp_AE

