//=========================================================================
//
//	FILE:				CP_DocumentWindow.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Window subclass for a window that is associated with
//						a CP_Document.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 194 $
//						Latest Checkout:  $Date: 2005-05-25 22:20:49 -0700 (Wed, 25 May 2005) $
//
//=========================================================================
#ifndef _H_CP_DocumentWindow
#define _H_CP_DocumentWindow
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_CommandHandler.h"
#include "CP_Window.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_Document_Base;

/*!
*	@class		CP_DocumentWindow
*	@brief		CP_Window subclass for a window that is associated with a CP_Document.
*	
*				This window class is designed to be used with a CP_Document so that the
*				data in the window can be saved, and closing the window will also cause the
*				documents save methods to be called automatically.
*
*				There should only be a single document window associated with a CP_Document,
*				all other windows should be reqular CP_Windows. Only a single document can
*				be associated with a document window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_DocumentWindow : public CP_Window
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DocumentWindow();
	//@}
	//! Destructor.
	virtual								~CP_DocumentWindow() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Return the CP_Document associated with this window.
										*	
										*	@exception	none
										*
										*	@return	CP_Document_Base *	The CP_Document_Base that is associated with this window.
										*/
			CP_Document_Base			*GetDocument()											{ return( fDocument ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the CP_Document associated with this window.
										*
										*	@param	inDocument		CP_Document_Base to associate with this window.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetDocument( CP_Document_Base *inDocument )				{ fDocument = inDocument; }
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );

	virtual void						OnClose();

	virtual	void						HandleSaveCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleSaveCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandleSaveAsCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleSaveAsCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandleRevertCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleRevertCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );

	virtual	void						HandleUndoCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleRedoCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleUndoCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandleRedoCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	// class data
protected:
	CP_Document_Base					*fDocument;		/*! CP_Document associated with this window. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DocumentWindow

