//=========================================================================
//
//	FILE:				CP_TableGeometry_Base.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Virtual base helper class for maining information about
//						the geometry in a CP_TableView class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 110 $
//						Latest Checkout:  $Date: 2005-03-23 14:37:56 -0800 (Wed, 23 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_TableGeometry_Base
#define _H_CP_TableGeometry_Base
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Object.h"
#include "CP_Point.h"
#include "CP_Rect.h"

CPLAT_Begin_Namespace_CPLAT

	// forward declarations
class CP_TableView;

const SInt16 kTableGeometry_DefaultRowHeight = 20;
const SInt16 kTableGeometry_DefaultColumnWidth = 50;

/*!
*	\class		CP_TableGeometry_Base
*	\brief		A helper class for CP_Tables that provides geometry information about table rows and columnns.
*/
class CP_EXPORT CP_TableGeometry_Base : public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TableGeometry_Base( CP_TableView *inTableView );
	//! Destructor.
	virtual								~CP_TableGeometry_Base() throw();
	//@}
		
	//! @name Columns
	//@{
	virtual SInt32						GetNumberColumns( SInt32 inRow ) = 0;
	virtual void						SetNumberColumns(
												SInt32 inRow,
												SInt32 inNumberColumns ) = 0;

	virtual SInt32						GetColumnWidth(
												SInt32 inRow,
												SInt32 inColumn ) = 0;
	virtual void						SetColumnWidth(
												SInt32 inRow,
												SInt32 inColumn,
												SInt32 inColumnWidth ) = 0;

	virtual	CP_Rect						GetColumnBounds(
												const CP_Rect& inRowBounds,
												SInt32 inRow,
												SInt32 inColumn ) = 0;
	virtual	CP_Rect						GetColumnBounds(
												SInt32 inRow,
												SInt32 inColumn ) = 0;
	//@}

	//! @name OwnerTableView
	//@{
			CP_TableView				*GetOwnerTable()									{ return( fOwnerTableView ); }
	//@}

	//! @name Rows
	//@{
	virtual SInt32						GetNumberRows() = 0;
	virtual void						SetNumberRows( SInt32 inNumberRows ) = 0;

	virtual SInt32						GetRowHeight( SInt32 inRow ) = 0;
	virtual void						SetRowHeight(
												SInt32 inRow,
												SInt32 inRowHeight ) = 0;

	virtual	CP_Point					GetRowLocation( SInt32 inRow ) = 0;

	virtual	void						AddRow() = 0;
	//@}

	//! @name Testers
	//@{
	virtual	SInt32						GetRowHitBy( const CP_Point& inHitPoint ) = 0;
	virtual	SInt32						GetColumnHitBy( const CP_Point& inHitPoint ) = 0;
	//@}

protected:
	//! @name Signals
	//@{
			void						SendRowAddedSignal();
			void						SendColumnAddedSignal();
			void						SendRowHeightChangedSignal( SInt32 inRow );
			void						SendColumnWidthChangedSignal( SInt32 inColumn );
	//@}

	// class data
protected:
	CP_TableView						*fOwnerTableView;

		// Signals
	CP_DataValueSignal					fRowHeightChangedSignal;
	CP_DataValueSignal					fColumnWidthChangedSignal;
	CP_DataValueSignal					fRowAddedSignal;
	CP_DataValueSignal					fColumnAddedSignal;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TableGeometry_Base

