//=========================================================================
//
//	FILE:				CP_MenuBarWindow.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for Windows or Linux to create a window that holds
//						a menu bar. This will provide an interface similar to
//						the MacOS with a single top level menu bar.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 160 $
//						Latest Checkout:  $Date: 2005-04-27 21:21:26 -0700 (Wed, 27 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_MenuBarWindow
#define _H_CP_MenuBarWindow
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32 || TARGET_OS_LINUX

	// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"


#include "CP_SignalSlot.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_MenuBarWindow : public CP_Window
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MenuBarWindow( CP_String& inTitle );

	//! Destructor.
	virtual								~CP_MenuBarWindow();
	//@}

	//! @name Instance
	//@{
										/*!
										*	Returns single instance of this class.
										*	
										*	@exception	none
										*	
										*	@return		CP_MenuBarWindow *		Returns the single instance of this class
										*/
	static 	CP_MenuBarWindow			*Instance()									{ return( fsInstance ); }
	//@}
	
	//! @name Setters
	//@{
			void						SetMenuBar( CP_MenuBar *inMenuBar );
	//@}

protected:
	//! @name Events
	//@{
			void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Initialization
	//@{
	virtual	void						CreatePlatformImp(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inWindowTitle,
												const CP_WindowTypeBase& inWindowType );
	//@}

private:
	//! @name Events
	//@{
			void						HandleCloseEvent( CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	static CP_MenuBarWindow				*fsInstance;		/*! The single instance of ourselves.*/

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_WIN32
public:
	//! @name Getters
	//@{
			OSWindow					GetOSWindow() const							{ return( fOSWindow ); }
	//@}


	//! @name Windows Messages
	//@{
	virtual	LRESULT						HandleWinProcMessage(
												HWND inHwnd,
												UINT inIMsg,
												WPARAM inWParam,
												LPARAM inLParam );
	//@}

private:
	//! @name Registration
	//@{
			void						RegisterMenuBarWindowClass();
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_WIN32 || TARGET_OS_LINUX

#endif	// _H_CP_MenuBarWindow
