//=========================================================================
//
//	FILE:				CP_View_Imp_Rich.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 194 $
//						Latest Checkout:  $Date: 2005-05-25 22:20:49 -0700 (Wed, 25 May 2005) $
//
//=========================================================================
#ifndef _H_CP_View_Imp_Rich
#define _H_CP_View_Imp_Rich
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_InStream.h"
#include "CP_KeyInputFilters.h"
#include "CP_M_CommandHandler.h"
#include "CP_View_Imp_Win.h"
	
CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_View_Imp_Rich : public CP_View_Imp_Win, public CP_Object
{
public:
											// Construction / destruction
										CP_View_Imp_Rich(
												CP_View_Imp *inOwnerViewImp,
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inReadOnly = false );
	virtual								~CP_View_Imp_Rich() throw();
			

	//! @name Appearance
	//@{
			void						Activate();
			void						Deactivate();
	//@}

	//! @name Command Handling
	//@{
			void						HandleCommandStatus(
												CP_CommandStatus& inCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommandNo );
										
			void						HandleCutCommand( OSStatus& inOutStatus );
			void						HandleCopyCommand( OSStatus& inOutStatus );
			void						HandleClearCommand( OSStatus& inOutStatus );
			void						HandlePasteCommand( OSStatus& inOutStatus );
			void						HandleSelectAllCommand( OSStatus& inOutStatus );
			void						HandleUndoCommand( OSStatus& inOutStatus );
			void						HandleRedoCommand( OSStatus& inOutStatus );
	//@}

	//! @name Drawing
	//@{
	virtual	void						HandleDraw( OSControl inControl );
	//@}

	//! @name Focus
	//@{
	virtual	void						OnLosingFocus();
	//@}

	//! @name Getters
	//@{
			OSStatus					ReadTextFromFile( const CP_FileSystemSpec& inFileSpec );
	//@}

	//! @name Getters
	//@{
			CP_String					GetText( char inLineEnd = CP_LF );
			UInt32						GetTextLength();
	//@}

	//! @name Key Filter
	//@{
			void						AttachFilter( CP_KeyInputFilter *inFilter );
			CP_KeyInputFilter			*GetFilter();
	//@}

	//! @name Printing
	//@{
			void						Print();
	//@}

	//! @name Selection
	//@{
			void						GetSelection(
												UInt32 *inStartSelection,
												UInt32 *inEndSelection );
			void						SetSelection(
												UInt32 inStartSelection,
												UInt32 inEndSelection );
			bool						HasSelection();
	//@}

	//! @name Setters
	//@{
			void						SetText( const CP_String& inText );
			void						SetText( CP_InStream *inStream );
			void						InsertText( const CP_String& inText );
			void						InsertText(
												const char *inTextPtr,
												SInt32 inTextLength );
			void						SetFontSpec( CP_FontSpec& inFontSpec );
	//@}

	//! @name Testers
	//@{
			bool						HasTextToPaste();
	//@}

protected:
	//! @name Setters
	//@{
	virtual	void						SetPortBounds( const CP_Rect& inBounds );

			void						SetTextFont( const CP_String& inFontName );
			void						SetTextSize( SInt16 inFontSize );
			void						SetTextFace( SInt16 inFace );
			void						SetTextColor( const CP_RGBColor& inForeColor );
	//@}

private:
	//! @name Events
	//@{
			void 						HandleKeyboardInput_Win( CP_Event_Base& inEvent );

			void						CommandProcess( CP_Event_Base& inEvent );
			void						CommandUpdateStatus( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						Init(
												CP_View *inSuperview,
												const CP_Point& inLocation,
												const CP_Size& inContentSize,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inReadOnly = false );
			void 						InitEventHandler( CPLAT::CP_EventHandler& inEventHandler );
			bool						HandleFieldExitChar( char inChar );
	//@}

	//! @name Misc
	//@{
			void 						TextChanged();
	//@}

	static	DWORD CALLBACK				CP_RichText_StreamInCallback(
												DWORD dwCookie,
												LPBYTE pbBuff,
												LONG cb,
												LONG *pcb );

	// class data
protected:
	CP_KeyInputFilter					*fKeyFilter;			// object used to filter our keystrokes

		// signals
	CP_DataValueSignal					fFieldExitSignal;		/*! Signal sent when the user enters a field exit key. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_View_Imp_Rich

